# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.currency_code import CurrencyCode
from openapi_client.models.adinsight.match_type import MatchType
from typing_extensions import Self

class EstimatedBidAndTraffic(BaseModel):
    """
    EstimatedBidAndTraffic
    """ # noqa: E501
    min_clicks_per_week: Optional[StrictFloat] = Field(default=None, alias="MinClicksPerWeek")
    max_clicks_per_week: Optional[StrictFloat] = Field(default=None, alias="MaxClicksPerWeek")
    average_cpc: Optional[StrictFloat] = Field(default=None, alias="AverageCPC")
    min_impressions_per_week: Optional[StrictStr] = Field(default=None, alias="MinImpressionsPerWeek")
    max_impressions_per_week: Optional[StrictStr] = Field(default=None, alias="MaxImpressionsPerWeek")
    ctr: Optional[StrictFloat] = Field(default=None, alias="CTR")
    min_total_cost_per_week: Optional[StrictFloat] = Field(default=None, alias="MinTotalCostPerWeek")
    max_total_cost_per_week: Optional[StrictFloat] = Field(default=None, alias="MaxTotalCostPerWeek")
    currency_code: Optional[CurrencyCode] = Field(default=None, alias="CurrencyCode")
    match_type: Optional[MatchType] = Field(default=None, alias="MatchType")
    estimated_min_bid: Optional[StrictFloat] = Field(default=None, alias="EstimatedMinBid")
    __properties: ClassVar[List[str]] = ["MinClicksPerWeek", "MaxClicksPerWeek", "AverageCPC", "MinImpressionsPerWeek", "MaxImpressionsPerWeek", "CTR", "MinTotalCostPerWeek", "MaxTotalCostPerWeek", "CurrencyCode", "MatchType", "EstimatedMinBid"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if min_clicks_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.min_clicks_per_week is None and "min_clicks_per_week" in self.model_fields_set:
            _dict['MinClicksPerWeek'] = None

        # set to None if max_clicks_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.max_clicks_per_week is None and "max_clicks_per_week" in self.model_fields_set:
            _dict['MaxClicksPerWeek'] = None

        # set to None if average_cpc (nullable) is None
        # and model_fields_set contains the field
        if self.average_cpc is None and "average_cpc" in self.model_fields_set:
            _dict['AverageCPC'] = None

        # set to None if min_impressions_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.min_impressions_per_week is None and "min_impressions_per_week" in self.model_fields_set:
            _dict['MinImpressionsPerWeek'] = None

        # set to None if max_impressions_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.max_impressions_per_week is None and "max_impressions_per_week" in self.model_fields_set:
            _dict['MaxImpressionsPerWeek'] = None

        # set to None if ctr (nullable) is None
        # and model_fields_set contains the field
        if self.ctr is None and "ctr" in self.model_fields_set:
            _dict['CTR'] = None

        # set to None if min_total_cost_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.min_total_cost_per_week is None and "min_total_cost_per_week" in self.model_fields_set:
            _dict['MinTotalCostPerWeek'] = None

        # set to None if max_total_cost_per_week (nullable) is None
        # and model_fields_set contains the field
        if self.max_total_cost_per_week is None and "max_total_cost_per_week" in self.model_fields_set:
            _dict['MaxTotalCostPerWeek'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if match_type (nullable) is None
        # and model_fields_set contains the field
        if self.match_type is None and "match_type" in self.model_fields_set:
            _dict['MatchType'] = None

        # set to None if estimated_min_bid (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_min_bid is None and "estimated_min_bid" in self.model_fields_set:
            _dict['EstimatedMinBid'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EstimatedBidAndTraffic from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "MinClicksPerWeek": obj.get("MinClicksPerWeek") if obj.get("MinClicksPerWeek") is not None else None,
                        "MaxClicksPerWeek": obj.get("MaxClicksPerWeek") if obj.get("MaxClicksPerWeek") is not None else None,
                        "AverageCPC": obj.get("AverageCPC") if obj.get("AverageCPC") is not None else None,
                        "MinImpressionsPerWeek": obj.get("MinImpressionsPerWeek") if obj.get("MinImpressionsPerWeek") is not None else None,
                        "MaxImpressionsPerWeek": obj.get("MaxImpressionsPerWeek") if obj.get("MaxImpressionsPerWeek") is not None else None,
                        "CTR": obj.get("CTR") if obj.get("CTR") is not None else None,
                        "MinTotalCostPerWeek": obj.get("MinTotalCostPerWeek") if obj.get("MinTotalCostPerWeek") is not None else None,
                        "MaxTotalCostPerWeek": obj.get("MaxTotalCostPerWeek") if obj.get("MaxTotalCostPerWeek") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "MatchType": obj.get("MatchType") if obj.get("MatchType") is not None else None,
                        "EstimatedMinBid": obj.get("EstimatedMinBid") if obj.get("EstimatedMinBid") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import _BulkAdExtensionBase
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

_CalloutAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('CalloutAdExtension'))


class BulkCalloutAdExtension(_BulkAdExtensionBase):
    """ Represents a callout ad extension.

    This class exposes the :attr:`callout_ad_extension` property that can be read and written
    as fields of the Callout Ad Extension record in a bulk file.

    For more information, see Callout Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _CalloutAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'CalloutAdExtension'
            ))
        super(BulkCalloutAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def callout_ad_extension(self):
        """ The callout ad extension.

        see Callout Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @callout_ad_extension.setter
    def callout_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.CalloutText,
            field_to_csv=lambda c: c.callout_ad_extension.Text,
            csv_to_field=lambda c, v: setattr(c.callout_ad_extension, 'Text', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.callout_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('CalloutAdExtension')
        self.callout_ad_extension.Type = 'CalloutAdExtension'
        super(BulkCalloutAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkCalloutAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.callout_ad_extension, 'callout_ad_extension')
        super(BulkCalloutAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkCalloutAdExtension._MAPPINGS)


class BulkAccountCalloutAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level callout ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Callout Ad Extension record in a bulk file.

    For more information, see Account Callout Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkCampaignCalloutAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level callout ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Callout Ad Extension record in a bulk file.

    For more information, see Campaign Callout Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupCalloutAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level Callout ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Callout Ad Extension record in a bulk file.

    For more information, see Ad Group Callout Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkCampaignBrandListAssociation(_SingleRecordBulkEntity):
    """ Represents a campaign brand list association.

    This class exposes the property :attr:`campaign_brand_list_association` that can be read and written as fields of the campaign brand list association record
    in a bulk file.

    For more information, see campaign brand list association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id=None, campaign_id=None, name=None, is_excluded=None):
        super(BulkCampaignBrandListAssociation, self).__init__()

        self._id = id
        self._campaign_id = campaign_id
        self._name = name
        self._is_excluded = is_excluded

    @property
    def id(self):
        """ The identifier of the campaign brand list association.

        Corresponds to the 'Id' field in the bulk file.

        :rtype: int
        """

        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @property
    def campaign_id(self):
        """ The identifier of the campaign that contains the brand list association.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._campaign_id

    @campaign_id.setter
    def campaign_id(self, campaign_id):
        self._campaign_id = campaign_id

    @property
    def name(self):
        """ The name of the brand list association.

        Corresponds to the 'Name' field in the bulk file.

        :rtype: str
        """

        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def is_excluded(self):
        """ Corresponds to the 'Is Excluded' field in the bulk file.

        :rtype: bool
        """

        return self._is_excluded

    @is_excluded.setter
    def is_excluded(self, is_excluded):
        self._is_excluded = is_excluded

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.name),
            csv_to_field=lambda c, v: setattr(c, 'name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.campaign_id),
            csv_to_field=lambda c, v: setattr(c, 'campaign_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IsExcluded,
            field_to_csv=lambda c: bulk_str(c.is_excluded),
            csv_to_field=lambda c, v: setattr(c, 'is_excluded', parse_bool(v))
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkCampaignBrandListAssociation._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkCampaignBrandListAssociation._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkCampaignBrandListAssociation, self).read_additional_data(stream_reader)



import re
from bingads.v13.internal.extensions import *

class _RowReportHeader:

    def __init__(self, stream_reader):
        self._regex = re.compile(r'^(Report Name|Report Time|Report Aggregation|Report Filter|Time Zone|Rows|Last Completed Available Day|Last Completed Available Hour|Potential Incomplete Data): (.*)$')
        self._value_regex = re.compile(r'^(([1-9]\d*\.?\d*)|(0\.\d*[1-9]))%?$')
        self._stream_reader = stream_reader
        self._report_name = None
        self._report_time_start = None
        self._report_time_end = None
        self._report_aggregation = None
        self._time_zone = None
        self._row_number = None
        self._last_completed_available_date = None
        self._report_columns = None
        self._potential_incomplete_data = None
        self.read_header()

    def read_header(self):
        self._stream_reader.read_report_header(self)
    
    def parse_header(self, fields):
        valid_values = list(filter(lambda x: x is not None and len(x) > 0, [s.strip() for s in fields]))
        if len(valid_values) == 1 and ':' in fields[0]:
            self.parse_meta(fields[0])
            return True
        else:
            for s in fields:
                if self._value_regex.match(s):
                    return False
            self._report_columns = fields
            return True
    
    def parse_meta(self, header):
        res = self._regex.match(header)
        if res:
            if res.group(1) == 'Report Name':
                self._report_name = res.group(2)
            elif res.group(1) == 'Report Time':
                self.set_report_time(res.group(2))
            elif res.group(1) == 'Report Aggregation':
                self._report_aggregation = res.group(2)
            elif res.group(1) == 'Report Filter':
                self._report_filter = res.group(2)
            elif res.group(1) == 'Time Zone':
                self._time_zone = res.group(2)
            elif res.group(1) == 'Rows':
                self._row_number = res.group(2)
            elif res.group(1) == 'Last Completed Available Day':
                self._last_completed_available_date = res.group(2)
            elif res.group(1) == 'Potential Incomplete Data':
                self._potential_incomplete_data = parse_bool(res.group(2))
        pass

    def set_report_time(self, report_time):
        if report_time is None or report_time == '':
            return
        time_array = report_time.split(',')

        if len(time_array) == 1:
            self._report_time_start = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
            self._report_time_end = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
        elif len(time_array) == 2:
            self._report_time_start = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
            self._report_time_end = datetime.strptime(time_array[1], '%m/%d/%Y') if time_array[1] else None

    @property
    def last_completed_available_date(self):
        if self._last_completed_available_date:
            str_time = self._last_completed_available_date
            if '(' in str_time:
                str_time = str_time[:str_time.rfind('(') - 1]

            try:
                return datetime.strptime(str_time, '%m/%d/%Y %I:%M:%S %p')
            except Exception:
                dt_tokens = str_time.split(' ')
                if len(dt_tokens) > 1:
                    return datetime.strptime(' '.join(dt_tokens[:2]), '%m/%d/%Y %H:%M:%S')
        return None
    
    @property
    def record_count(self):
        return int(self._row_number) if self._row_number else None
    
    
    @property
    def time_zone(self):
        return self._time_zone
    
    @property
    def potential_incomplete_data(self):
        return self._potential_incomplete_data
    
#    @property
#    def report_filter(self):
#        return self._report_filter
    
    @property
    def report_aggregation(self):
        return self._report_aggregation
    
    @property
    def report_name(self):
        return self._report_name
    
    @property
    def report_time_start(self):
        return self._report_time_start

    @property
    def report_time_end(self):
        return self._report_time_end

    @property
    def report_columns(self):
        return self._report_columns




from bingads.v13.bulk.entities import QualityScoreData
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *

def coop_setting_to_csv(bulk_ad_group, row_values):
    if not bulk_ad_group.ad_group.Settings or not bulk_ad_group.ad_group.Settings.Setting:
        return
    settings = [setting for setting in bulk_ad_group.ad_group.Settings.Setting if isinstance(setting, CoOpSetting_Type)]
    if len(settings) == 0:
        return
    if len(settings) != 1:
        raise ValueError('Can only have 1 CoOpSetting in AdGroup Settings.')

    row_values[_StringTable.MaximumBid] = settings[0].BidMaxValue
    row_values[_StringTable.BidBoostValue] = settings[0].BidBoostValue
    row_values[_StringTable.BidOption] = settings[0].BidOption
    pass

def csv_to_coop_setting(row_values, bulk_ad_group):
    maximum_bid_success, maximum_bid = row_values.try_get_value(_StringTable.MaximumBid)
    bid_boost_value_success, bid_boost_value = row_values.try_get_value(_StringTable.BidBoostValue)
    bid_option_success, bid_option = row_values.try_get_value(_StringTable.BidOption)

    if maximum_bid_success or bid_boost_value_success or bid_option_success:
        coop_setting = _CAMPAIGN_OBJECT_FACTORY_V13.create('CoOpSetting')
        coop_setting.Type = 'CoOpSetting'
        coop_setting.BidOption = parse_bid_option(bid_option)
        coop_setting.BidBoostValue = float(bid_boost_value) if bid_boost_value else None
        coop_setting.BidMaxValue = float(maximum_bid) if maximum_bid else None
        bulk_ad_group.ad_group.Settings.Setting.append(coop_setting)
    pass

def bidding_scheme_to_csv(bulk_ad_group, row_values):
    bid_strategy_type = field_to_csv_BidStrategyType(bulk_ad_group.ad_group)
    if not bid_strategy_type:
        return
    row_values[_StringTable.BidStrategyType] = bid_strategy_type
    if bid_strategy_type == 'InheritFromParent' \
        and hasattr(bulk_ad_group.ad_group.BiddingScheme, 'InheritedBidStrategyType'):
        row_values[_StringTable.InheritedBidStrategyType] = bulk_ad_group.ad_group.BiddingScheme.InheritedBidStrategyType


def csv_to_bidding_scheme(row_values, bulk_ad_group):
    success, bid_strategy_type = row_values.try_get_value(_StringTable.BidStrategyType)
    if not success or not bid_strategy_type:
        return
    csv_to_field_BidStrategyType(bulk_ad_group.ad_group, bid_strategy_type)
    if bid_strategy_type == 'InheritFromParent':
        bulk_ad_group.ad_group.BiddingScheme.Type = "InheritFromParent"
        success, inherited_bid_strategy_type = row_values.try_get_value(_StringTable.InheritedBidStrategyType)
        if success and inherited_bid_strategy_type != '':
            bulk_ad_group.ad_group.BiddingScheme.InheritedBidStrategyType = inherited_bid_strategy_type
        elif hasattr(bulk_ad_group.ad_group.BiddingScheme, 'InheritedBidStrategyType'):
            bulk_ad_group.ad_group.BiddingScheme.InheritedBidStrategyType = None
    else:
        bulk_ad_group.ad_group.BiddingScheme.Type = bid_strategy_type


class BulkAdGroup(_SingleRecordBulkEntity):
    """ Represents an ad group.

    This class exposes the property :attr:`ad_group` that can be read and written as fields of the Ad Group record
    in a bulk file.

    For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, campaign_id=None, campaign_name=None, ad_group=None):
        super(BulkAdGroup, self).__init__()

        self._campaign_id = campaign_id
        self._campaign_name = campaign_name
        self._ad_group = ad_group

        self._quality_score_data = None
        self._performance_data = None


    @property
    def campaign_id(self):
        """ The identifier of the campaign that contains the ad group.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._campaign_id

    @campaign_id.setter
    def campaign_id(self, campaign_id):
        self._campaign_id = campaign_id

    @property
    def campaign_name(self):
        """ The name of the campaign that contains the ad group.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def ad_group(self):
        """ The AdGroup Data Object of the Campaign Management Service.

        A subset of AdGroup properties are available in the Ad Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._ad_group

    @ad_group.setter
    def ad_group(self, ad_group):
        self._ad_group = ad_group

    @property
    def quality_score_data(self):
        """ The quality score data for the ad group.

        :rtype: QualityScoreData
        """
        return self._quality_score_data

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.ad_group.Id),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.ad_group.Status),
            csv_to_field=csv_to_status
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.campaign_id),
            csv_to_field=lambda c, v: setattr(c, 'campaign_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: c.ad_group.Name,
            csv_to_field=lambda c, v: setattr(c.ad_group, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: bulk_date_str(c.ad_group.StartDate),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'StartDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: bulk_date_str(c.ad_group.EndDate),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'EndDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.NetworkDistribution,
            field_to_csv=lambda c: bulk_str(c.ad_group.Network),
            csv_to_field=lambda c, v: csv_to_field_enum(c.ad_group, v, 'Network', Network)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdRotation,
            field_to_csv=lambda c: ad_rotation_bulk_str(c.ad_group.AdRotation, c.ad_group.Id),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'AdRotation', parse_ad_rotation(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.CpcBid,
            field_to_csv=lambda c: ad_group_bid_bulk_str(c.ad_group.CpcBid),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'CpcBid', parse_ad_group_bid(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.CpvBid,
            field_to_csv=lambda c: ad_group_bid_bulk_str(c.ad_group.CpvBid),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'CpvBid', parse_ad_group_bid(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.CpmBid,
            field_to_csv=lambda c: ad_group_bid_bulk_str(c.ad_group.CpmBid),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'CpmBid', parse_ad_group_bid(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.Language,
            field_to_csv=lambda c: bulk_str(c.ad_group.Language),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'Language', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.BidAdjustment,
            field_to_csv=lambda c: bulk_str(c.ad_group.AudienceAdsBidAdjustment),
            csv_to_field=lambda c, v: setattr(
                c.ad_group,
                'AudienceAdsBidAdjustment',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_optional_str(c.ad_group.TrackingUrlTemplate, c.ad_group.Id),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.ad_group),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.ad_group, v)
        ),
        _ComplexBulkMapping(bidding_scheme_to_csv, csv_to_bidding_scheme),

        _SimpleBulkMapping(
            header=_StringTable.TargetSetting,
            field_to_csv=lambda c: target_setting_to_csv(c.ad_group),
            csv_to_field=lambda c, v: csv_to_target_setting(c.ad_group, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PrivacyStatus,
            field_to_csv=lambda c: bulk_str(c.ad_group.PrivacyStatus),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'PrivacyStatus', v if v else None)
        ),

        _ComplexBulkMapping(coop_setting_to_csv, csv_to_coop_setting),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.ad_group.FinalUrlSuffix, c.ad_group.Id),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'FinalUrlSuffix', v)
        ),

        _SimpleBulkMapping(
            header=_StringTable.AdScheduleUseSearcherTimeZone,
            field_to_csv=lambda c: field_to_csv_UseSearcherTimeZone(c.ad_group.AdScheduleUseSearcherTimeZone, None),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'AdScheduleUseSearcherTimeZone', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroupType,
            field_to_csv=lambda c: c.ad_group.AdGroupType,
            csv_to_field=lambda c, v: setattr(c.ad_group, 'AdGroupType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.MultiMediaAdBidAdjustment,
            field_to_csv=lambda c: bulk_str(c.ad_group.MultimediaAdsBidAdjustment),
            csv_to_field=lambda c, v: setattr(
                c.ad_group,
                'MultimediaAdsBidAdjustment',
                int(v) if v else None
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.UseOptimizedTargeting,
            field_to_csv=lambda c: field_to_csv_bool(c.ad_group.UseOptimizedTargeting),
            csv_to_field=lambda c, v: setattr(c.ad_group, 'UseOptimizedTargeting', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.HotelAdGroupType,
            field_to_csv=lambda c: hotel_setting_to_csv(c.ad_group),
            csv_to_field=lambda c, v: csv_to_hotel_setting(c.ad_group, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CommissionRate,
            field_to_csv=lambda c: bulk_str(c.ad_group.CommissionRate.RateAmount.Amount),
            csv_to_field=lambda c, v: csv_to_commission_rate(c.ad_group, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PercentCpcBid,
            field_to_csv=lambda c: bulk_str(c.ad_group.PercentCpcBid.RateAmount.Amount),
            csv_to_field=lambda c, v: csv_to_percent_cpc_bid(c.ad_group, v)
        )
    ]


    def process_mappings_from_row_values(self, row_values):
        self.ad_group = _CAMPAIGN_OBJECT_FACTORY_V13.create('AdGroup')

        row_values.convert_to_entity(self, BulkAdGroup._MAPPINGS)

        self._quality_score_data = QualityScoreData.read_from_row_values_or_null(row_values)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._ad_group, 'AdGroup')
        self.convert_to_values(row_values, BulkAdGroup._MAPPINGS)
        if not exclude_readonly_data:
            QualityScoreData.write_to_row_values_if_not_null(self.quality_score_data, row_values)

    def read_additional_data(self, stream_reader):
        super(BulkAdGroup, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CombinationPerformanceReportColumn(str, Enum):
    TIMEPERIOD = 'TimePeriod'

    ACCOUNTID = 'AccountId'

    ACCOUNTNAME = 'AccountName'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNTYPE = 'CampaignType'

    ADGROUPID = 'AdGroupId'

    ADGROUPNAME = 'AdGroupName'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ADID = 'AdId'

    ADTYPE = 'AdType'

    HEADLINE1 = 'Headline1'

    HEADLINE2 = 'Headline2'

    HEADLINE3 = 'Headline3'

    COMBINATIONLONGHEADLINE = 'CombinationLongHeadline'

    DESCRIPTION1 = 'Description1'

    DESCRIPTION2 = 'Description2'

    IMAGE = 'Image'

    LOGO = 'Logo'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    SPEND = 'Spend'

    CTR = 'Ctr'

    CONVERSIONS = 'Conversions'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CombinationPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.campaign.action_ad_extension import ActionAdExtension
from openapi_client.models.campaign.action_ad_extension_action_type import ActionAdExtensionActionType
from openapi_client.models.campaign.ad_extension_header_type import AdExtensionHeaderType
from openapi_client.models.campaign.ad_extension_status import AdExtensionStatus
from openapi_client.models.campaign.address import Address
from openapi_client.models.campaign.app_ad_extension import AppAdExtension
from openapi_client.models.campaign.app_url import AppUrl
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.business_geo_code_status import BusinessGeoCodeStatus
from openapi_client.models.campaign.call_ad_extension import CallAdExtension
from openapi_client.models.campaign.callout_ad_extension import CalloutAdExtension
from openapi_client.models.campaign.confirmation_action import ConfirmationAction
from openapi_client.models.campaign.custom_parameters import CustomParameters
from openapi_client.models.campaign.data_table_ad_extension import DataTableAdExtension
from openapi_client.models.campaign.disclaimer_ad_extension import DisclaimerAdExtension
from openapi_client.models.campaign.filter_link_ad_extension import FilterLinkAdExtension
from openapi_client.models.campaign.flyer_ad_extension import FlyerAdExtension
from openapi_client.models.campaign.geo_point import GeoPoint
from openapi_client.models.campaign.image_ad_extension import ImageAdExtension
from openapi_client.models.campaign.image_asset import ImageAsset
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.lead_delivery import LeadDelivery
from openapi_client.models.campaign.lead_form_ad_extension import LeadFormAdExtension
from openapi_client.models.campaign.lead_form_call_to_action import LeadFormCallToAction
from openapi_client.models.campaign.lead_form_question_definition import LeadFormQuestionDefinition
from openapi_client.models.campaign.location_ad_extension import LocationAdExtension
from openapi_client.models.campaign.logo_ad_extension import LogoAdExtension
from openapi_client.models.campaign.model_date import ModelDate
from openapi_client.models.campaign.news_ad_extension import NewsAdExtension
from openapi_client.models.campaign.price_ad_extension import PriceAdExtension
from openapi_client.models.campaign.price_extension_type import PriceExtensionType
from openapi_client.models.campaign.price_table_row import PriceTableRow
from openapi_client.models.campaign.promotion_ad_extension import PromotionAdExtension
from openapi_client.models.campaign.promotion_discount_modifier import PromotionDiscountModifier
from openapi_client.models.campaign.promotion_occasion import PromotionOccasion
from openapi_client.models.campaign.review_ad_extension import ReviewAdExtension
from openapi_client.models.campaign.schedule import Schedule
from openapi_client.models.campaign.sitelink_ad_extension import SitelinkAdExtension
from openapi_client.models.campaign.structured_snippet_ad_extension import StructuredSnippetAdExtension
from openapi_client.models.campaign.video_ad_extension import VideoAdExtension
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

AdExtension_ONE_OF_SCHEMAS = ["ActionAdExtension", "AppAdExtension", "CallAdExtension", "CalloutAdExtension", "DataTableAdExtension", "DisclaimerAdExtension", "FilterLinkAdExtension", "FlyerAdExtension", "ImageAdExtension", "LeadFormAdExtension", "LocationAdExtension", "LogoAdExtension", "NewsAdExtension", "PriceAdExtension", "PromotionAdExtension", "ReviewAdExtension", "SitelinkAdExtension", "StructuredSnippetAdExtension", "VideoAdExtension"]

class AdExtension(BaseModel):
    """
    AdExtension
    """
    # data type: ActionAdExtension
    oneof_schema_action_ad_extension_validator: Optional[ActionAdExtension] = None
    # data type: AppAdExtension
    oneof_schema_app_ad_extension_validator: Optional[AppAdExtension] = None
    # data type: CallAdExtension
    oneof_schema_call_ad_extension_validator: Optional[CallAdExtension] = None
    # data type: CalloutAdExtension
    oneof_schema_callout_ad_extension_validator: Optional[CalloutAdExtension] = None
    # data type: DataTableAdExtension
    oneof_schema_data_table_ad_extension_validator: Optional[DataTableAdExtension] = None
    # data type: DisclaimerAdExtension
    oneof_schema_disclaimer_ad_extension_validator: Optional[DisclaimerAdExtension] = None
    # data type: FilterLinkAdExtension
    oneof_schema_filter_link_ad_extension_validator: Optional[FilterLinkAdExtension] = None
    # data type: FlyerAdExtension
    oneof_schema_flyer_ad_extension_validator: Optional[FlyerAdExtension] = None
    # data type: ImageAdExtension
    oneof_schema_image_ad_extension_validator: Optional[ImageAdExtension] = None
    # data type: LeadFormAdExtension
    oneof_schema_lead_form_ad_extension_validator: Optional[LeadFormAdExtension] = None
    # data type: LocationAdExtension
    oneof_schema_location_ad_extension_validator: Optional[LocationAdExtension] = None
    # data type: LogoAdExtension
    oneof_schema_logo_ad_extension_validator: Optional[LogoAdExtension] = None
    # data type: NewsAdExtension
    oneof_schema_news_ad_extension_validator: Optional[NewsAdExtension] = None
    # data type: PriceAdExtension
    oneof_schema_price_ad_extension_validator: Optional[PriceAdExtension] = None
    # data type: PromotionAdExtension
    oneof_schema_promotion_ad_extension_validator: Optional[PromotionAdExtension] = None
    # data type: ReviewAdExtension
    oneof_schema_review_ad_extension_validator: Optional[ReviewAdExtension] = None
    # data type: SitelinkAdExtension
    oneof_schema_sitelink_ad_extension_validator: Optional[SitelinkAdExtension] = None
    # data type: StructuredSnippetAdExtension
    oneof_schema_structured_snippet_ad_extension_validator: Optional[StructuredSnippetAdExtension] = None
    # data type: VideoAdExtension
    oneof_schema_video_ad_extension_validator: Optional[VideoAdExtension] = None
    actual_instance: Optional[Union[ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension]] = None
    one_of_schemas: Set[str] = { "ActionAdExtension", "AppAdExtension", "CallAdExtension", "CalloutAdExtension", "DataTableAdExtension", "DisclaimerAdExtension", "FilterLinkAdExtension", "FlyerAdExtension", "ImageAdExtension", "LeadFormAdExtension", "LocationAdExtension", "LogoAdExtension", "NewsAdExtension", "PriceAdExtension", "PromotionAdExtension", "ReviewAdExtension", "SitelinkAdExtension", "StructuredSnippetAdExtension", "VideoAdExtension" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = AdExtension.model_construct()
        error_messages = []
        match = 0
        # validate data type: ActionAdExtension
        if not isinstance(v, ActionAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActionAdExtension`")
        else:
            match += 1
        # validate data type: AppAdExtension
        if not isinstance(v, AppAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AppAdExtension`")
        else:
            match += 1
        # validate data type: CallAdExtension
        if not isinstance(v, CallAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CallAdExtension`")
        else:
            match += 1
        # validate data type: CalloutAdExtension
        if not isinstance(v, CalloutAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CalloutAdExtension`")
        else:
            match += 1
        # validate data type: DataTableAdExtension
        if not isinstance(v, DataTableAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DataTableAdExtension`")
        else:
            match += 1
        # validate data type: DisclaimerAdExtension
        if not isinstance(v, DisclaimerAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DisclaimerAdExtension`")
        else:
            match += 1
        # validate data type: FilterLinkAdExtension
        if not isinstance(v, FilterLinkAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `FilterLinkAdExtension`")
        else:
            match += 1
        # validate data type: FlyerAdExtension
        if not isinstance(v, FlyerAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `FlyerAdExtension`")
        else:
            match += 1
        # validate data type: ImageAdExtension
        if not isinstance(v, ImageAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ImageAdExtension`")
        else:
            match += 1
        # validate data type: LeadFormAdExtension
        if not isinstance(v, LeadFormAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LeadFormAdExtension`")
        else:
            match += 1
        # validate data type: LocationAdExtension
        if not isinstance(v, LocationAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocationAdExtension`")
        else:
            match += 1
        # validate data type: LogoAdExtension
        if not isinstance(v, LogoAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LogoAdExtension`")
        else:
            match += 1
        # validate data type: NewsAdExtension
        if not isinstance(v, NewsAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `NewsAdExtension`")
        else:
            match += 1
        # validate data type: PriceAdExtension
        if not isinstance(v, PriceAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PriceAdExtension`")
        else:
            match += 1
        # validate data type: PromotionAdExtension
        if not isinstance(v, PromotionAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PromotionAdExtension`")
        else:
            match += 1
        # validate data type: ReviewAdExtension
        if not isinstance(v, ReviewAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ReviewAdExtension`")
        else:
            match += 1
        # validate data type: SitelinkAdExtension
        if not isinstance(v, SitelinkAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SitelinkAdExtension`")
        else:
            match += 1
        # validate data type: StructuredSnippetAdExtension
        if not isinstance(v, StructuredSnippetAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `StructuredSnippetAdExtension`")
        else:
            match += 1
        # validate data type: VideoAdExtension
        if not isinstance(v, VideoAdExtension):
            error_messages.append(f"Error! Input type `{type(v)}` is not `VideoAdExtension`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in AdExtension with oneOf schemas: ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in AdExtension with oneOf schemas: ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `ActionAdExtension`
        if _data_type == "ActionAdExtension":
            instance.actual_instance = ActionAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `AppAdExtension`
        if _data_type == "AppAdExtension":
            instance.actual_instance = AppAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `CallAdExtension`
        if _data_type == "CallAdExtension":
            instance.actual_instance = CallAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `CalloutAdExtension`
        if _data_type == "CalloutAdExtension":
            instance.actual_instance = CalloutAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `DataTableAdExtension`
        if _data_type == "DataTableAdExtension":
            instance.actual_instance = DataTableAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `DisclaimerAdExtension`
        if _data_type == "DisclaimerAdExtension":
            instance.actual_instance = DisclaimerAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `FilterLinkAdExtension`
        if _data_type == "FilterLinkAdExtension":
            instance.actual_instance = FilterLinkAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `FlyerAdExtension`
        if _data_type == "FlyerAdExtension":
            instance.actual_instance = FlyerAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `ImageAdExtension`
        if _data_type == "ImageAdExtension":
            instance.actual_instance = ImageAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `LeadFormAdExtension`
        if _data_type == "LeadFormAdExtension":
            instance.actual_instance = LeadFormAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `LocationAdExtension`
        if _data_type == "LocationAdExtension":
            instance.actual_instance = LocationAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `LogoAdExtension`
        if _data_type == "LogoAdExtension":
            instance.actual_instance = LogoAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `NewsAdExtension`
        if _data_type == "NewsAdExtension":
            instance.actual_instance = NewsAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `PriceAdExtension`
        if _data_type == "PriceAdExtension":
            instance.actual_instance = PriceAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `PromotionAdExtension`
        if _data_type == "PromotionAdExtension":
            instance.actual_instance = PromotionAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `ReviewAdExtension`
        if _data_type == "ReviewAdExtension":
            instance.actual_instance = ReviewAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `SitelinkAdExtension`
        if _data_type == "SitelinkAdExtension":
            instance.actual_instance = SitelinkAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `StructuredSnippetAdExtension`
        if _data_type == "StructuredSnippetAdExtension":
            instance.actual_instance = StructuredSnippetAdExtension.from_json(json_str)
            return instance
			
		# check if data type is `VideoAdExtension`
        if _data_type == "VideoAdExtension":
            instance.actual_instance = VideoAdExtension.from_json(json_str)
            return instance
			

        # deserialize data into ActionAdExtension
        try:
            instance.actual_instance = ActionAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AppAdExtension
        try:
            instance.actual_instance = AppAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CallAdExtension
        try:
            instance.actual_instance = CallAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CalloutAdExtension
        try:
            instance.actual_instance = CalloutAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DataTableAdExtension
        try:
            instance.actual_instance = DataTableAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DisclaimerAdExtension
        try:
            instance.actual_instance = DisclaimerAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into FilterLinkAdExtension
        try:
            instance.actual_instance = FilterLinkAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into FlyerAdExtension
        try:
            instance.actual_instance = FlyerAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ImageAdExtension
        try:
            instance.actual_instance = ImageAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LeadFormAdExtension
        try:
            instance.actual_instance = LeadFormAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocationAdExtension
        try:
            instance.actual_instance = LocationAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LogoAdExtension
        try:
            instance.actual_instance = LogoAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into NewsAdExtension
        try:
            instance.actual_instance = NewsAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PriceAdExtension
        try:
            instance.actual_instance = PriceAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PromotionAdExtension
        try:
            instance.actual_instance = PromotionAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ReviewAdExtension
        try:
            instance.actual_instance = ReviewAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SitelinkAdExtension
        try:
            instance.actual_instance = SitelinkAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into StructuredSnippetAdExtension
        try:
            instance.actual_instance = StructuredSnippetAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into VideoAdExtension
        try:
            instance.actual_instance = VideoAdExtension.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into AdExtension with oneOf schemas: ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into AdExtension with oneOf schemas: ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], ActionAdExtension, AppAdExtension, CallAdExtension, CalloutAdExtension, DataTableAdExtension, DisclaimerAdExtension, FilterLinkAdExtension, FlyerAdExtension, ImageAdExtension, LeadFormAdExtension, LocationAdExtension, LogoAdExtension, NewsAdExtension, PriceAdExtension, PromotionAdExtension, ReviewAdExtension, SitelinkAdExtension, StructuredSnippetAdExtension, VideoAdExtension]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_action_ad_extension_validator', 'oneof_schema_app_ad_extension_validator', 'oneof_schema_call_ad_extension_validator', 'oneof_schema_callout_ad_extension_validator', 'oneof_schema_data_table_ad_extension_validator', 'oneof_schema_disclaimer_ad_extension_validator', 'oneof_schema_filter_link_ad_extension_validator', 'oneof_schema_flyer_ad_extension_validator', 'oneof_schema_image_ad_extension_validator', 'oneof_schema_lead_form_ad_extension_validator', 'oneof_schema_location_ad_extension_validator', 'oneof_schema_logo_ad_extension_validator', 'oneof_schema_news_ad_extension_validator', 'oneof_schema_price_ad_extension_validator', 'oneof_schema_promotion_ad_extension_validator', 'oneof_schema_review_ad_extension_validator', 'oneof_schema_sitelink_ad_extension_validator', 'oneof_schema_structured_snippet_ad_extension_validator', 'oneof_schema_video_ad_extension_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_audience import BulkAudience

class BulkProductAudience(BulkAudience):
    """ Represents a Product Audience that can be read or written in a bulk file.

    This class exposes the :attr:`product_audience` property that can be read and written as fields of the
    Product Audience record in a bulk file.

    For more information, see Product Audience at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 product_audience=None,
                 status=None,):
        super(BulkProductAudience, self).__init__(audience = BulkAudience, status = status)

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.TagId,
            field_to_csv=lambda c: bulk_str(c.product_audience.TagId),
            csv_to_field=lambda c, v: setattr(c.product_audience, 'TagId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            _StringTable.ProductAudienceType,
            field_to_csv=lambda c: bulk_str(c.product_audience.ProductAudienceType),
            csv_to_field=lambda c, v: setattr(c.product_audience, 'ProductAudienceType', v)
        ),
    ]

    @property
    def product_audience(self):
        """ Defines a Product Audience """

        return self._audience

    @product_audience.setter
    def product_audience(self, product_audience):
        self._audience = product_audience


    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.product_audience, 'product_audience')
        super(BulkProductAudience, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkProductAudience._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self.product_audience = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProductAudience')
        super(BulkProductAudience, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkProductAudience._MAPPINGS)




# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class PriceExtensionType(str, Enum):
    UNKNOWN = 'Unknown'

    BRANDS = 'Brands'

    EVENTS = 'Events'

    LOCATIONS = 'Locations'

    NEIGHBORHOODS = 'Neighborhoods'

    PRODUCTCATEGORIES = 'ProductCategories'

    PRODUCTTIERS = 'ProductTiers'

    SERVICES = 'Services'

    SERVICECATEGORIES = 'ServiceCategories'

    SERVICETIERS = 'ServiceTiers'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PriceExtensionType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class VanityPharmaWebsiteDescription(str, Enum):
    UNSPECIFIED = 'Unspecified'

    PRESCRIPTION_TREATMENT_WEBSITE_EN = 'PrescriptionTreatmentWebsite_En'

    PRESCRIPTION_TREATMENT_WEBSITE_ES = 'PrescriptionTreatmentWebsite_Es'

    PRESCRIPTION_DEVICE_WEBSITE_EN = 'PrescriptionDeviceWebsite_En'

    PRESCRIPTION_DEVICE_WEBSITE_ES = 'PrescriptionDeviceWebsite_Es'

    MEDICAL_DEVICE_WEBSITE_EN = 'MedicalDeviceWebsite_En'

    MEDICAL_DEVICE_WEBSITE_ES = 'MedicalDeviceWebsite_Es'

    PREVENTATIVE_TREATMENT_WEBSITE_EN = 'PreventativeTreatmentWebsite_En'

    PREVENTATIVE_TREATMENT_WEBSITE_ES = 'PreventativeTreatmentWebsite_Es'

    PRESCRIPTION_CONTRACEPTION_WEBSITE_EN = 'PrescriptionContraceptionWebsite_En'

    PRESCRIPTION_CONTRACEPTION_WEBSITE_ES = 'PrescriptionContraceptionWebsite_Es'

    PRESCRIPTION_VACCINE_WEBSITE_EN = 'PrescriptionVaccineWebsite_En'

    PRESCRIPTION_VACCINE_WEBSITE_ES = 'PrescriptionVaccineWebsite_Es'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of VanityPharmaWebsiteDescription from a JSON string"""
        return cls(json.loads(json_str))





try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

VERSION = '13.0.24.2b3'

with open('README.rst', 'r') as f:
    readme = f.read()
with open('HISTORY.rst', 'r') as f:
    history = f.read().replace('.. :changelog:', '')

requirements = [
    'suds-community>=1.1.0',
    'requests',
    'enum34;python_version<"3.9"',
    'pydantic>=2.0.0',
    'pydantic-core>=2.0.1',
    'typing-extensions>=4.0.0'
]

setup(
    name='msads',
    version=VERSION,
    description='A library to make working with the Bing Ads APIs and bulk services easy',
    long_description=readme,
    author='Bing Ads SDK Team',
    author_email='bing_ads_sdk@microsoft.com',
    url='https://github.com/BingAds/BingAds-Python-SDK',
    packages=[
        'bingads',
        'bingads.internal',
        'bingads.v13',
        'bingads.v13.bulk',
        'bingads.v13.bulk.entities',
        'bingads.v13.bulk.entities.ad_extensions',
        'bingads.v13.bulk.entities.audiences',
        'bingads.v13.bulk.entities.feeds',
        'bingads.v13.bulk.entities.target_criterions',
        'bingads.v13.bulk.entities.labels',
        'bingads.v13.bulk.entities.goals',
        'bingads.v13.internal',
        'bingads.v13.internal.bulk',
        'bingads.v13.internal.bulk.entities',
        'bingads.v13.internal.reporting',
        'bingads.v13.reporting',
        'openapi_client',
        'openapi_client.api',
        'openapi_client.models',
        'openapi_client.models.bulk',
        'openapi_client.models.campaign',
        'openapi_client.models.reporting',
        'openapi_client.models.customer',
        'openapi_client.models.billing',
        'openapi_client.models.adinsight',
    ],
    include_package_data=True,
    install_requires=requirements,
    license='MIT',
    zip_safe=False,
    keywords='bingads',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
    ],
)



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping, _ComplexBulkMapping
from bingads.v13.internal.extensions import *

class BulkAssetGroup(_SingleRecordBulkEntity):
    """ Represents an asset group.

    This class exposes the property :attr:`asset_group` that can be read and written as fields of the Asset Group record
    in a bulk file.

    For more information, see Asset Group at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, campaign_id=None, campaign_name=None, asset_group=None):
        super(BulkAssetGroup, self).__init__()

        self._campaign_id = campaign_id
        self._campaign_name = campaign_name
        self._asset_group = asset_group


    @property
    def campaign_id(self):
        """ The identifier of the campaign that contains the asset group.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._campaign_id

    @campaign_id.setter
    def campaign_id(self, campaign_id):
        self._campaign_id = campaign_id

    @property
    def campaign_name(self):
        """ The name of the campaign that contains the asset group.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def asset_group(self):
        """ The AssetGroup Data Object of the Campaign Management Service.

        A subset of AssetGroup properties are available in the Ad Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._asset_group

    @asset_group.setter
    def asset_group(self, asset_group):
        self._asset_group = asset_group

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.asset_group.Id),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.asset_group.Status),
            csv_to_field=lambda c, v: csv_to_field_enum(c.asset_group, v, 'Status', AssetGroupStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.campaign_id),
            csv_to_field=lambda c, v: setattr(c, 'campaign_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, 'campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AssetGroup,
            field_to_csv=lambda c: bulk_str(c.asset_group.Name),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: bulk_date_str(c.asset_group.StartDate),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'StartDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: bulk_date_str(c.asset_group.EndDate),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'EndDate', parse_date(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.BusinessName,
            field_to_csv=lambda c: c.asset_group.BusinessName,
            csv_to_field=lambda c, v: setattr(c.asset_group, 'BusinessName', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CallToAction,
            field_to_csv=lambda c: c.asset_group.CallToAction,
            csv_to_field=lambda c, v: csv_to_field_enum(c.asset_group, v, 'CallToAction', CallToAction)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Descriptions,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.asset_group.Descriptions),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.asset_group.Descriptions ,v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: c.asset_group.EditorialStatus,
            csv_to_field=lambda c, v: csv_to_field_enum(c.asset_group, v, 'EditorialStatus', AssetGroupEditorialStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.asset_group.FinalMobileUrls, c.asset_group.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.asset_group.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.asset_group.FinalUrls, c.asset_group.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.asset_group.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Headlines,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.asset_group.Headlines),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.asset_group.Headlines, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Images,
            field_to_csv=lambda c: field_to_csv_ImageAssetLinks(c.asset_group.Images),
            csv_to_field=lambda c, v: csv_to_field_ImageAssetLinks(c.asset_group.Images, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LongHeadlines,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.asset_group.LongHeadlines),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.asset_group.LongHeadlines ,v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path1,
            field_to_csv=lambda c: bulk_optional_str(c.asset_group.Path1, c.asset_group.Id),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'Path1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path2,
            field_to_csv=lambda c: bulk_optional_str(c.asset_group.Path2, c.asset_group.Id),
            csv_to_field=lambda c, v: setattr(c.asset_group, 'Path2', v)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        self.asset_group = _CAMPAIGN_OBJECT_FACTORY_V13.create('AssetGroup')

        row_values.convert_to_entity(self, BulkAssetGroup._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._asset_group, 'AssetGroup')
        self.convert_to_values(row_values, BulkAssetGroup._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAssetGroup, self).read_additional_data(stream_reader)



from bingads.v13.bulk.entities import *
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from decimal import Decimal

class BulkConversionGoal(_SingleRecordBulkEntity):
    """ Represents a ConversionGoal that can be read or written in a bulk file.

    This class exposes the :attr:`conversion_goal` property that can be read and written as fields of the
    ConversionGoal record in a bulk file.

    For more information, see ConversionGoal at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 conversion_goal=None):
        super(BulkConversionGoal, self).__init__()

        self._conversion_goal = conversion_goal

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Status),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'Status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Id),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Name),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'Name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AttributionModelType,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.AttributionModelType),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'AttributionModelType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ConversionWindowInMinutes,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.ConversionWindowInMinutes),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'ConversionWindowInMinutes', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CountType,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.CountType),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'CountType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ExcludeFromBidding,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.ExcludeFromBidding),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'ExcludeFromBidding', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.GoalCategory,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.GoalCategory),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'GoalCategory', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.IsEnhancedConversionsEnabled,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.IsEnhancedConversionsEnabled),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'IsEnhancedConversionsEnabled', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.CurrencyCode,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Revenue.CurrencyCode),
            csv_to_field=lambda c, v: setattr(c.conversion_goal.Revenue, 'CurrencyCode', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.RevenueValue,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Revenue.Value),
            csv_to_field=lambda c, v: setattr(c.conversion_goal.Revenue, 'Value', Decimal(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.RevenueType,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Revenue.Type),
            csv_to_field=lambda c, v: setattr(c.conversion_goal.Revenue, 'Type', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Scope,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.Scope),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'Scope', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TagId,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.TagId),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'TagId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ViewThroughConversionWindowInMinutes,
            field_to_csv=lambda c: bulk_str(c.conversion_goal.ViewThroughConversionWindowInMinutes),
            csv_to_field=lambda c, v: setattr(c.conversion_goal, 'ViewThroughConversionWindowInMinutes', int(v) if v else None)
        ),
    ]

    @property
    def conversion_goal(self):
        """ Defines a ConversionGoal """

        return self._conversion_goal

    @conversion_goal.setter
    def conversion_goal(self, conversion_goal):
        self._conversion_goal = conversion_goal

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkConversionGoal._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkConversionGoal._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkConversionGoal, self).read_additional_data(stream_reader)



import codecs
import xml.etree.cElementTree as et
import re
from bingads.v13.internal.extensions import *
from bingads.v13.reporting.report_contract import *

'''
Implement with context.next.xml.etree.cElementTree is far more faster than xml.etree.ElementTree with python 2.7.10
'''

schema=''
class _XmlReportRecord:
    def __init__(self, row):
        self._row_values = {}
        start = len(schema)
        for elem in row:
            self._row_values[elem.tag[start:]] = elem.attrib['value']

    def value(self, header):
        if header in self._row_values:
            return self._row_values[header]
        raise InvalidReportColumnException(header)

    def int_value(self, header):
        return 0 if '--' == self.value(header) else int(self.value(header))

    def double_value(self, header):
        return 0.0 if '--' == self.value(header) else float(self.value(header))

class _XmlReportIterator():
    
    def __init__(self, context, header):
        self._context = context
        self._header = header
        self._row_pattern = schema + 'Row'
        self._next_report_record = None
        self.move_to_next_row()
                 
    def move_to_next_row(self):
        for event, elem in self._context:
            if event == 'end' and self._row_pattern == elem.tag:
                self._next_report_record = _XmlReportRecord(elem)
                # self.try_set_report_columns(elem)
                elem.clear
                break
    def try_set_report_columns(self, element):
        if len(self._header._report_columns) == 0:
            start = len(schema)
            for e in element:
                self._header._report_columns.append(e.tag[start:])
        

    def __enter__(self):
        return self

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()
    
    def next(self):
        if self._next_report_record == None:
            raise StopIteration()
            
        ret = self._next_report_record
        self._next_report_record = None
        self.move_to_next_row()
        return ret


class _XmlReportHeader:

    def __init__(self, context):
        self._report_columns = []
        if context is not None:
            event, root = context.next()
            if event == 'start':
                m = re.compile(r'^(.*)Report').match(root.tag)
                if not m:
                    raise InvalidReportContentException()
                self.set_report_attribute(root.attrib.copy())
                global schema
                schema = m.group(1)
                root.clear()
            self.parse_column_names(context)
    
    def set_report_attribute(self, report_attr):
        self._report_attr = report_attr
        self.set_report_time()

    def set_report_time(self):
        self._report_time_start = None
        self._report_time_end = None
        if 'ReportTime' in self._report_attr:
            time_array = self._report_attr['ReportTime'].split(',')
            
            if len(time_array) == 1:
                self._report_time_start = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
                self._report_time_end = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
            elif len(time_array) == 2:
                self._report_time_start = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[0] else None
                self._report_time_end = datetime.strptime(time_array[0], '%m/%d/%Y') if time_array[1] else None
    
    def parse_column_names(self, context):
        for event, elem in context:
            if event == 'start' and schema + 'Column' == elem.tag:
                self._report_columns.append(elem.attrib['name'])
                elem.clear()
            elif  event == 'start' and 'Table' in elem.tag:
                # in case there is no columns names - ExcludeColumnHeader is set to true.
                break

    @property
    def last_completed_available_date(self):
        if 'LastCompletedAvailableDay'  in self._report_attr:
            str_time = self._report_attr['LastCompletedAvailableDay']
            if '(' in str_time:
                str_time = str_time[:str_time.rfind('(') - 1]
        
            try:
                return datetime.strptime(str_time, '%m/%d/%Y %I:%M:%S %p')
            except Exception:
                dt_tokens = str_time.split(' ')
                if len(dt_tokens) > 1:
                    return datetime.strptime(' '.join(dt_tokens[:2]), '%m/%d/%Y %H:%M:%S')
        return None
    
    @property
    def record_count(self):
        return int(self._report_attr['Rows']) if 'Rows' in self._report_attr else None
    
    @property
    def time_zone(self):
        return self._report_attr['TimeZone'] if 'TimeZone' in self._report_attr else None
    
#    @property
#    def report_filter(self):
#        return None
    
    @property
    def report_aggregation(self):
        return self._report_attr['ReportAggregation'] if 'ReportAggregation' in self._report_attr else None
    
    @property
    def report_name(self):
        return self._report_attr['ReportName'] if 'ReportName' in self._report_attr else None
    
    @property
    def potential_incomplete_data(self):
        return parse_bool(self._report_attr['PotentialIncompleteData']) if 'PotentialIncompleteData' in self._report_attr else None

    @property
    def report_time_start(self):
        return self._report_time_start
    
    @property
    def report_time_end(self):
        return self._report_time_end
    
    @property
    def report_columns(self):
        return self._report_columns
    
        

class _XmlReport(Report):
    
    def __init__(self, file):
        context = et.iterparse(file, events=("start", "end"))
        self._report_header = _XmlReportHeader(context)
        self._report_iterator = _XmlReportIterator(context, self._report_header)



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class Currency(str, Enum):
    UNSPECIFY = 'UnSpecify'

    AUSTRALIANDOLLAR = 'AustralianDollar'

    BRAZILIANREAL = 'BrazilianReal'

    CANADIANDOLLAR = 'CanadianDollar'

    SWISSFRANC = 'SwissFranc'

    DANISHKRONE = 'DanishKrone'

    EURO = 'EURO'

    UKPOUND = 'UKPound'

    JAPANESEYEN = 'JapaneseYen'

    NORWEGIANKRONE = 'NorwegianKrone'

    NEWZEALANDDOLLAR = 'NewZealandDollar'

    SWEDISHKRONA = 'SwedishKrona'

    USDOLLAR = 'USDollar'

    SOUTHAFRICANRAND = 'SouthAfricanRand'

    ARGENTINEPESO = 'ArgentinePeso'

    CHILEANPESO = 'ChileanPeso'

    YUANRENMINBI = 'YuanRenminbi'

    COLOMBIANPESO = 'ColombianPeso'

    CZECHKORUNA = 'CzechKoruna'

    UAEDIRHAM = 'UAEDirham'

    ALGERIANDINAR = 'AlgerianDinar'

    EGYPTIANPOUND = 'EgyptianPound'

    HONGKONGDOLLAR = 'HongKongDollar'

    FORINT = 'Forint'

    RUPIAH = 'Rupiah'

    ISRAELISHEQEL = 'IsraeliSheqel'

    INDIANRUPEE = 'IndianRupee'

    WON = 'Won'

    LEBANESEPOUND = 'LebanesePound'

    MOROCCANDIRHAM = 'MoroccanDirham'

    MEXICANPESO = 'MexicanPeso'

    MALAYSIANRINGGIT = 'MalaysianRinggit'

    NUEVOSOL = 'NuevoSol'

    PHILIPPINEPESO = 'PhilippinePeso'

    POLISHZLOTY = 'PolishZloty'

    QATARIRIAL = 'QatariRial'

    SAUDIRIYAL = 'SaudiRiyal'

    SINGAPOREDOLLAR = 'SingaporeDollar'

    BAHT = 'Baht'

    NEWTAIWANDOLLAR = 'NewTaiwanDollar'

    BOLIVAR = 'Bolivar'

    DONG = 'Dong'

    TURKISHLIRA = 'TurkishLira'

    NIGERIANNAIRA = 'NigerianNaira'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Currency from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.brand_kit_font import BrandKitFont
from openapi_client.models.campaign.brand_kit_image import BrandKitImage
from openapi_client.models.campaign.brand_kit_palette import BrandKitPalette
from typing_extensions import Self

class BrandKit(BaseModel):
    """
    BrandKit
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    business_name: Optional[StrictStr] = Field(default=None, alias="BusinessName")
    images: Optional[List[Optional[BrandKitImage]]] = Field(default=None, alias="Images")
    square_logos: Optional[List[Optional[BrandKitImage]]] = Field(default=None, alias="SquareLogos")
    landscape_logos: Optional[List[Optional[BrandKitImage]]] = Field(default=None, alias="LandscapeLogos")
    palettes: Optional[List[Optional[BrandKitPalette]]] = Field(default=None, alias="Palettes")
    fonts: Optional[List[Optional[BrandKitFont]]] = Field(default=None, alias="Fonts")
    __properties: ClassVar[List[str]] = ["Id", "Name", "BusinessName", "Images", "SquareLogos", "LandscapeLogos", "Palettes", "Fonts"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in images (list)
        _items = []
        if self.images:
            for _item_images in self.images:
                if _item_images:
                    _items.append(_item_images.to_dict())
            _dict['Images'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in square_logos (list)
        _items = []
        if self.square_logos:
            for _item_square_logos in self.square_logos:
                if _item_square_logos:
                    _items.append(_item_square_logos.to_dict())
            _dict['SquareLogos'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in landscape_logos (list)
        _items = []
        if self.landscape_logos:
            for _item_landscape_logos in self.landscape_logos:
                if _item_landscape_logos:
                    _items.append(_item_landscape_logos.to_dict())
            _dict['LandscapeLogos'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in palettes (list)
        _items = []
        if self.palettes:
            for _item_palettes in self.palettes:
                if _item_palettes:
                    _items.append(_item_palettes.to_dict())
            _dict['Palettes'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in fonts (list)
        _items = []
        if self.fonts:
            for _item_fonts in self.fonts:
                if _item_fonts:
                    _items.append(_item_fonts.to_dict())
            _dict['Fonts'] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if business_name (nullable) is None
        # and model_fields_set contains the field
        if self.business_name is None and "business_name" in self.model_fields_set:
            _dict['BusinessName'] = None

        # set to None if images (nullable) is None
        # and model_fields_set contains the field
        if self.images is None and "images" in self.model_fields_set:
            _dict['Images'] = None

        # set to None if square_logos (nullable) is None
        # and model_fields_set contains the field
        if self.square_logos is None and "square_logos" in self.model_fields_set:
            _dict['SquareLogos'] = None

        # set to None if landscape_logos (nullable) is None
        # and model_fields_set contains the field
        if self.landscape_logos is None and "landscape_logos" in self.model_fields_set:
            _dict['LandscapeLogos'] = None

        # set to None if palettes (nullable) is None
        # and model_fields_set contains the field
        if self.palettes is None and "palettes" in self.model_fields_set:
            _dict['Palettes'] = None

        # set to None if fonts (nullable) is None
        # and model_fields_set contains the field
        if self.fonts is None and "fonts" in self.model_fields_set:
            _dict['Fonts'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BrandKit from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "BusinessName": obj.get("BusinessName") if obj.get("BusinessName") is not None else None,
                        "Images": [BrandKitImage.from_dict(_item) for _item in obj["Images"]] if obj.get("Images") is not None else None,
                        "SquareLogos": [BrandKitImage.from_dict(_item) for _item in obj["SquareLogos"]] if obj.get("SquareLogos") is not None else None,
                        "LandscapeLogos": [BrandKitImage.from_dict(_item) for _item in obj["LandscapeLogos"]] if obj.get("LandscapeLogos") is not None else None,
                        "Palettes": [BrandKitPalette.from_dict(_item) for _item in obj["Palettes"]] if obj.get("Palettes") is not None else None,
                        "Fonts": [BrandKitFont.from_dict(_item) for _item in obj["Fonts"]] if obj.get("Fonts") is not None else None
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class KeywordDemographic(BaseModel):
    """
    KeywordDemographic
    """ # noqa: E501
    device: Optional[StrictStr] = Field(default=None, alias="Device")
    eighteen_to_twenty_four: Optional[StrictFloat] = Field(default=None, alias="EighteenToTwentyFour")
    twenty_five_to_thirty_four: Optional[StrictFloat] = Field(default=None, alias="TwentyFiveToThirtyFour")
    thirty_five_to_fourty_nine: Optional[StrictFloat] = Field(default=None, alias="ThirtyFiveToFourtyNine")
    fifty_to_sixty_four: Optional[StrictFloat] = Field(default=None, alias="FiftyToSixtyFour")
    sixty_five_and_above: Optional[StrictFloat] = Field(default=None, alias="SixtyFiveAndAbove")
    age_unknown: Optional[StrictFloat] = Field(default=None, alias="AgeUnknown")
    female: Optional[StrictFloat] = Field(default=None, alias="Female")
    male: Optional[StrictFloat] = Field(default=None, alias="Male")
    gender_unknown: Optional[StrictFloat] = Field(default=None, alias="GenderUnknown")
    __properties: ClassVar[List[str]] = ["Device", "EighteenToTwentyFour", "TwentyFiveToThirtyFour", "ThirtyFiveToFourtyNine", "FiftyToSixtyFour", "SixtyFiveAndAbove", "AgeUnknown", "Female", "Male", "GenderUnknown"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if device (nullable) is None
        # and model_fields_set contains the field
        if self.device is None and "device" in self.model_fields_set:
            _dict['Device'] = None

        # set to None if eighteen_to_twenty_four (nullable) is None
        # and model_fields_set contains the field
        if self.eighteen_to_twenty_four is None and "eighteen_to_twenty_four" in self.model_fields_set:
            _dict['EighteenToTwentyFour'] = None

        # set to None if twenty_five_to_thirty_four (nullable) is None
        # and model_fields_set contains the field
        if self.twenty_five_to_thirty_four is None and "twenty_five_to_thirty_four" in self.model_fields_set:
            _dict['TwentyFiveToThirtyFour'] = None

        # set to None if thirty_five_to_fourty_nine (nullable) is None
        # and model_fields_set contains the field
        if self.thirty_five_to_fourty_nine is None and "thirty_five_to_fourty_nine" in self.model_fields_set:
            _dict['ThirtyFiveToFourtyNine'] = None

        # set to None if fifty_to_sixty_four (nullable) is None
        # and model_fields_set contains the field
        if self.fifty_to_sixty_four is None and "fifty_to_sixty_four" in self.model_fields_set:
            _dict['FiftyToSixtyFour'] = None

        # set to None if sixty_five_and_above (nullable) is None
        # and model_fields_set contains the field
        if self.sixty_five_and_above is None and "sixty_five_and_above" in self.model_fields_set:
            _dict['SixtyFiveAndAbove'] = None

        # set to None if age_unknown (nullable) is None
        # and model_fields_set contains the field
        if self.age_unknown is None and "age_unknown" in self.model_fields_set:
            _dict['AgeUnknown'] = None

        # set to None if female (nullable) is None
        # and model_fields_set contains the field
        if self.female is None and "female" in self.model_fields_set:
            _dict['Female'] = None

        # set to None if male (nullable) is None
        # and model_fields_set contains the field
        if self.male is None and "male" in self.model_fields_set:
            _dict['Male'] = None

        # set to None if gender_unknown (nullable) is None
        # and model_fields_set contains the field
        if self.gender_unknown is None and "gender_unknown" in self.model_fields_set:
            _dict['GenderUnknown'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of KeywordDemographic from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Device": obj.get("Device") if obj.get("Device") is not None else None,
                        "EighteenToTwentyFour": obj.get("EighteenToTwentyFour") if obj.get("EighteenToTwentyFour") is not None else None,
                        "TwentyFiveToThirtyFour": obj.get("TwentyFiveToThirtyFour") if obj.get("TwentyFiveToThirtyFour") is not None else None,
                        "ThirtyFiveToFourtyNine": obj.get("ThirtyFiveToFourtyNine") if obj.get("ThirtyFiveToFourtyNine") is not None else None,
                        "FiftyToSixtyFour": obj.get("FiftyToSixtyFour") if obj.get("FiftyToSixtyFour") is not None else None,
                        "SixtyFiveAndAbove": obj.get("SixtyFiveAndAbove") if obj.get("SixtyFiveAndAbove") is not None else None,
                        "AgeUnknown": obj.get("AgeUnknown") if obj.get("AgeUnknown") is not None else None,
                        "Female": obj.get("Female") if obj.get("Female") is not None else None,
                        "Male": obj.get("Male") if obj.get("Male") is not None else None,
                        "GenderUnknown": obj.get("GenderUnknown") if obj.get("GenderUnknown") is not None else None
        })
        return _obj



# coding: utf-8

"""
    BulkService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class DownloadEntity(str, Enum):
    NONE = 'None'

    CAMPAIGNS = 'Campaigns'

    ADGROUPS = 'AdGroups'

    ADS = 'Ads'

    KEYWORDS = 'Keywords'

    CAMPAIGNNEGATIVEKEYWORDS = 'CampaignNegativeKeywords'

    ADGROUPNEGATIVEKEYWORDS = 'AdGroupNegativeKeywords'

    CAMPAIGNTARGETCRITERIONS = 'CampaignTargetCriterions'

    ADGROUPTARGETCRITERIONS = 'AdGroupTargetCriterions'

    CAMPAIGNNEGATIVESITES = 'CampaignNegativeSites'

    ADGROUPNEGATIVESITES = 'AdGroupNegativeSites'

    CAMPAIGNLOCATIONADEXTENSIONS = 'CampaignLocationAdExtensions'

    CAMPAIGNCALLADEXTENSIONS = 'CampaignCallAdExtensions'

    LOCATIONADEXTENSIONS = 'LocationAdExtensions'

    CALLADEXTENSIONS = 'CallAdExtensions'

    NEGATIVEKEYWORDLISTS = 'NegativeKeywordLists'

    SHAREDNEGATIVEKEYWORDS = 'SharedNegativeKeywords'

    CAMPAIGNNEGATIVEKEYWORDLISTASSOCIATIONS = 'CampaignNegativeKeywordListAssociations'

    IMAGEADEXTENSIONS = 'ImageAdExtensions'

    CAMPAIGNIMAGEADEXTENSIONS = 'CampaignImageAdExtensions'

    ADGROUPIMAGEADEXTENSIONS = 'AdGroupImageAdExtensions'

    APPADEXTENSIONS = 'AppAdExtensions'

    ADGROUPAPPADEXTENSIONS = 'AdGroupAppAdExtensions'

    CAMPAIGNAPPADEXTENSIONS = 'CampaignAppAdExtensions'

    PRICEADEXTENSIONS = 'PriceAdExtensions'

    REVIEWADEXTENSIONS = 'ReviewAdExtensions'

    CAMPAIGNNEGATIVEDYNAMICSEARCHADTARGETS = 'CampaignNegativeDynamicSearchAdTargets'

    ADGROUPPRODUCTPARTITIONS = 'AdGroupProductPartitions'

    CAMPAIGNPRODUCTSCOPES = 'CampaignProductScopes'

    CAMPAIGNREVIEWADEXTENSIONS = 'CampaignReviewAdExtensions'

    ADGROUPREVIEWADEXTENSIONS = 'AdGroupReviewAdExtensions'

    CALLOUTADEXTENSIONS = 'CalloutAdExtensions'

    CAMPAIGNCALLOUTADEXTENSIONS = 'CampaignCalloutAdExtensions'

    ADGROUPCALLOUTADEXTENSIONS = 'AdGroupCalloutAdExtensions'

    SITELINKADEXTENSIONS = 'SitelinkAdExtensions'

    CAMPAIGNSITELINKADEXTENSIONS = 'CampaignSitelinkAdExtensions'

    ADGROUPSITELINKADEXTENSIONS = 'AdGroupSitelinkAdExtensions'

    STRUCTUREDSNIPPETADEXTENSIONS = 'StructuredSnippetAdExtensions'

    CAMPAIGNSTRUCTUREDSNIPPETADEXTENSIONS = 'CampaignStructuredSnippetAdExtensions'

    ADGROUPSTRUCTUREDSNIPPETADEXTENSIONS = 'AdGroupStructuredSnippetAdExtensions'

    REMARKETINGLISTS = 'RemarketingLists'

    ADGROUPREMARKETINGLISTASSOCIATIONS = 'AdGroupRemarketingListAssociations'

    BUDGETS = 'Budgets'

    TEXTADS = 'TextAds'

    PRODUCTADS = 'ProductAds'

    APPINSTALLADS = 'AppInstallAds'

    EXPANDEDTEXTADS = 'ExpandedTextAds'

    DYNAMICSEARCHADS = 'DynamicSearchAds'

    ADGROUPDYNAMICSEARCHADTARGETS = 'AdGroupDynamicSearchAdTargets'

    ADGROUPNEGATIVEDYNAMICSEARCHADTARGETS = 'AdGroupNegativeDynamicSearchAdTargets'

    CAMPAIGNPRICEADEXTENSIONS = 'CampaignPriceAdExtensions'

    ADGROUPPRICEADEXTENSIONS = 'AdGroupPriceAdExtensions'

    LABELS = 'Labels'

    CAMPAIGNLABELS = 'CampaignLabels'

    ADGROUPLABELS = 'AdGroupLabels'

    TEXTADLABELS = 'TextAdLabels'

    KEYWORDLABELS = 'KeywordLabels'

    ADGROUPNEGATIVEREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeRemarketingListAssociations'

    CUSTOMAUDIENCES = 'CustomAudiences'

    ADGROUPCUSTOMAUDIENCEASSOCIATIONS = 'AdGroupCustomAudienceAssociations'

    ADGROUPNEGATIVECUSTOMAUDIENCEASSOCIATIONS = 'AdGroupNegativeCustomAudienceAssociations'

    INMARKETAUDIENCES = 'InMarketAudiences'

    ADGROUPINMARKETAUDIENCEASSOCIATIONS = 'AdGroupInMarketAudienceAssociations'

    ADGROUPNEGATIVEINMARKETAUDIENCEASSOCIATIONS = 'AdGroupNegativeInMarketAudienceAssociations'

    AUDIENCES = 'Audiences'

    ADGROUPAUDIENCEASSOCIATIONS = 'AdGroupAudienceAssociations'

    ADGROUPNEGATIVEAUDIENCEASSOCIATIONS = 'AdGroupNegativeAudienceAssociations'

    PRODUCTADLABELS = 'ProductAdLabels'

    APPINSTALLADLABELS = 'AppInstallAdLabels'

    EXPANDEDTEXTADLABELS = 'ExpandedTextAdLabels'

    DYNAMICSEARCHADLABELS = 'DynamicSearchAdLabels'

    ACCOUNTLOCATIONADEXTENSIONS = 'AccountLocationAdExtensions'

    ACCOUNTIMAGEADEXTENSIONS = 'AccountImageAdExtensions'

    ACCOUNTAPPADEXTENSIONS = 'AccountAppAdExtensions'

    ACCOUNTPRICEADEXTENSIONS = 'AccountPriceAdExtensions'

    ACCOUNTREVIEWADEXTENSIONS = 'AccountReviewAdExtensions'

    ACCOUNTCALLOUTADEXTENSIONS = 'AccountCalloutAdExtensions'

    ACCOUNTSITELINKADEXTENSIONS = 'AccountSitelinkAdExtensions'

    ACCOUNTSTRUCTUREDSNIPPETADEXTENSIONS = 'AccountStructuredSnippetAdExtensions'

    RESPONSIVEADS = 'ResponsiveAds'

    RESPONSIVEADLABELS = 'ResponsiveAdLabels'

    PRODUCTAUDIENCES = 'ProductAudiences'

    ADGROUPPRODUCTAUDIENCEASSOCIATIONS = 'AdGroupProductAudienceAssociations'

    ADGROUPNEGATIVEPRODUCTAUDIENCEASSOCIATIONS = 'AdGroupNegativeProductAudienceAssociations'

    SIMILARREMARKETINGLISTS = 'SimilarRemarketingLists'

    ADGROUPSIMILARREMARKETINGLISTASSOCIATIONS = 'AdGroupSimilarRemarketingListAssociations'

    ADGROUPNEGATIVESIMILARREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeSimilarRemarketingListAssociations'

    EXPERIMENTS = 'Experiments'

    ACTIONADEXTENSIONS = 'ActionAdExtensions'

    CAMPAIGNACTIONADEXTENSIONS = 'CampaignActionAdExtensions'

    ADGROUPACTIONADEXTENSIONS = 'AdGroupActionAdExtensions'

    ACCOUNTACTIONADEXTENSIONS = 'AccountActionAdExtensions'

    RESPONSIVESEARCHADS = 'ResponsiveSearchAds'

    RESPONSIVESEARCHADLABELS = 'ResponsiveSearchAdLabels'

    CAMPAIGNAUDIENCEASSOCIATIONS = 'CampaignAudienceAssociations'

    CAMPAIGNNEGATIVEAUDIENCEASSOCIATIONS = 'CampaignNegativeAudienceAssociations'

    CAMPAIGNCUSTOMAUDIENCEASSOCIATIONS = 'CampaignCustomAudienceAssociations'

    CAMPAIGNNEGATIVECUSTOMAUDIENCEASSOCIATIONS = 'CampaignNegativeCustomAudienceAssociations'

    CAMPAIGNINMARKETAUDIENCEASSOCIATIONS = 'CampaignInMarketAudienceAssociations'

    CAMPAIGNNEGATIVEINMARKETAUDIENCEASSOCIATIONS = 'CampaignNegativeInMarketAudienceAssociations'

    CAMPAIGNPRODUCTAUDIENCEASSOCIATIONS = 'CampaignProductAudienceAssociations'

    CAMPAIGNNEGATIVEPRODUCTAUDIENCEASSOCIATIONS = 'CampaignNegativeProductAudienceAssociations'

    CAMPAIGNREMARKETINGLISTASSOCIATIONS = 'CampaignRemarketingListAssociations'

    CAMPAIGNNEGATIVEREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeRemarketingListAssociations'

    CAMPAIGNSIMILARREMARKETINGLISTASSOCIATIONS = 'CampaignSimilarRemarketingListAssociations'

    CAMPAIGNNEGATIVESIMILARREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeSimilarRemarketingListAssociations'

    FEEDS = 'Feeds'

    FEEDITEMS = 'FeedItems'

    CAMPAIGNNEGATIVESTORECRITERIONS = 'CampaignNegativeStoreCriterions'

    PROMOTIONADEXTENSIONS = 'PromotionAdExtensions'

    ACCOUNTPROMOTIONADEXTENSIONS = 'AccountPromotionAdExtensions'

    CAMPAIGNPROMOTIONADEXTENSIONS = 'CampaignPromotionAdExtensions'

    ADGROUPPROMOTIONADEXTENSIONS = 'AdGroupPromotionAdExtensions'

    COMBINEDLISTS = 'CombinedLists'

    ADGROUPCOMBINEDLISTASSOCIATIONS = 'AdGroupCombinedListAssociations'

    ADGROUPNEGATIVECOMBINEDLISTASSOCIATIONS = 'AdGroupNegativeCombinedListAssociations'

    CAMPAIGNCOMBINEDLISTASSOCIATIONS = 'CampaignCombinedListAssociations'

    CAMPAIGNNEGATIVECOMBINEDLISTASSOCIATIONS = 'CampaignNegativeCombinedListAssociations'

    IMAGES = 'Images'

    CUSTOMERLISTS = 'CustomerLists'

    ADGROUPCUSTOMERLISTASSOCIATIONS = 'AdGroupCustomerListAssociations'

    ADGROUPNEGATIVECUSTOMERLISTASSOCIATIONS = 'AdGroupNegativeCustomerListAssociations'

    CAMPAIGNCUSTOMERLISTASSOCIATIONS = 'CampaignCustomerListAssociations'

    CAMPAIGNNEGATIVECUSTOMERLISTASSOCIATIONS = 'CampaignNegativeCustomerListAssociations'

    FILTERLINKADEXTENSIONS = 'FilterLinkAdExtensions'

    ACCOUNTFILTERLINKADEXTENSIONS = 'AccountFilterLinkAdExtensions'

    CAMPAIGNFILTERLINKADEXTENSIONS = 'CampaignFilterLinkAdExtensions'

    ADGROUPFILTERLINKADEXTENSIONS = 'AdGroupFilterLinkAdExtensions'

    FLYERADEXTENSIONS = 'FlyerAdExtensions'

    ACCOUNTFLYERADEXTENSIONS = 'AccountFlyerAdExtensions'

    CAMPAIGNFLYERADEXTENSIONS = 'CampaignFlyerAdExtensions'

    ADGROUPFLYERADEXTENSIONS = 'AdGroupFlyerAdExtensions'

    BIDSTRATEGIES = 'BidStrategies'

    VIDEOADEXTENSIONS = 'VideoAdExtensions'

    ACCOUNTVIDEOADEXTENSIONS = 'AccountVideoAdExtensions'

    CAMPAIGNVIDEOADEXTENSIONS = 'CampaignVideoAdExtensions'

    ADGROUPVIDEOADEXTENSIONS = 'AdGroupVideoAdExtensions'

    VIDEOS = 'Videos'

    DISCLAIMERADEXTENSIONS = 'DisclaimerAdExtensions'

    CAMPAIGNDISCLAIMERADEXTENSIONS = 'CampaignDisclaimerAdExtensions'

    ADCUSTOMIZERATTRIBUTE = 'AdcustomizerAttribute'

    CAMPAIGNADCUSTOMIZERATTRIBUTE = 'CampaignAdcustomizerAttribute'

    ADGROUPADCUSTOMIZERATTRIBUTE = 'AdGroupAdcustomizerAttribute'

    KEYWORDADCUSTOMIZERATTRIBUTE = 'KeywordAdcustomizerAttribute'

    CAMPAIGNCONVERSIONGOAL = 'CampaignConversionGoal'

    ADGROUPHOTELLISTINGGROUPS = 'AdGroupHotelListingGroups'

    ASSETGROUPS = 'AssetGroups'

    AUDIENCEGROUPS = 'AudienceGroups'

    CAMPAIGNNEGATIVEWEBPAGES = 'CampaignNegativeWebpages'

    AUDIENCEGROUPASSETGROUPASSOCIATIONS = 'AudienceGroupAssetGroupAssociations'

    ASSETGROUPLISTINGGROUPS = 'AssetGroupListingGroups'

    LOGOADEXTENSIONS = 'LogoAdExtensions'

    ACCOUNTLOGOADEXTENSIONS = 'AccountLogoAdExtensions'

    CAMPAIGNLOGOADEXTENSIONS = 'CampaignLogoAdExtensions'

    ADGROUPLOGOADEXTENSIONS = 'AdGroupLogoAdExtensions'

    LEADFORMADEXTENSIONS = 'LeadFormAdExtensions'

    ACCOUNTLEADFORMADEXTENSIONS = 'AccountLeadFormAdExtensions'

    CAMPAIGNLEADFORMADEXTENSIONS = 'CampaignLeadFormAdExtensions'

    ACCOUNTNEGATIVEKEYWORDLIST = 'AccountNegativeKeywordList'

    ACCOUNTNEGATIVEKEYWORDLISTASSOCIATION = 'AccountNegativeKeywordListAssociation'

    ACCOUNTNEGATIVEKEYWORD = 'AccountNegativeKeyword'

    CONVERSIONGOAL = 'ConversionGoal'

    EVENTGOAL = 'EventGoal'

    APPINSTALLGOAL = 'AppInstallGoal'

    MULTISTAGEGOAL = 'MultiStageGoal'

    DURATIONGOAL = 'DurationGoal'

    OFFLINECONVERSIONGOAL = 'OfflineConversionGoal'

    URLGOAL = 'UrlGoal'

    INSTORETRANSACTIONGOAL = 'InStoreTransactionGoal'

    PAGESVIEWEDPERVISITGOAL = 'PagesViewedPerVisitGoal'

    SMARTGOAL = 'SmartGoal'

    INSTOREVISITGOAL = 'InStoreVisitGoal'

    PRODUCTGOAL = 'ProductGoal'

    SEASONALITYADJUSTMENTS = 'SeasonalityAdjustments'

    DATAEXCLUSIONS = 'DataExclusions'

    BRANDLIST = 'BrandList'

    BRANDITEM = 'BrandItem'

    CAMPAIGNBRANDLISTASSOCIATION = 'CampaignBrandListAssociation'

    ASSETGROUPSEARCHTHEMES = 'AssetGroupSearchThemes'

    IMPRESSIONBASEDREMARKETINGLIST = 'ImpressionBasedRemarketingList'

    ADGROUPIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'AdGroupImpressionBasedRemarketingListAssociations'

    ADGROUPNEGATIVEIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'AdGroupNegativeImpressionBasedRemarketingListAssociations'

    CAMPAIGNIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'CampaignImpressionBasedRemarketingListAssociations'

    CAMPAIGNNEGATIVEIMPRESSIONBASEDREMARKETINGLISTASSOCIATIONS = 'CampaignNegativeImpressionBasedRemarketingListAssociations'

    ASSETGROUPURLTARGETS = 'AssetGroupUrlTargets'

    NEWCUSTOMERACQUISITIONGOAL = 'NewCustomerAcquisitionGoal'

    CONVERSIONVALUERULE = 'ConversionValueRule'

    ACCOUNTPLACEMENTEXCLUSIONLIST = 'AccountPlacementExclusionList'

    ACCOUNTPLACEMENTEXCLUSIONLISTITEM = 'AccountPlacementExclusionListItem'

    CAMPAIGNACCOUNTPLACEMENTEXCLUSIONLISTASSOCIATION = 'CampaignAccountPlacementExclusionListAssociation'

    ACCOUNTPLACEMENTINCLUSIONLIST = 'AccountPlacementInclusionList'

    ACCOUNTPLACEMENTINCLUSIONLISTITEM = 'AccountPlacementInclusionListItem'

    CAMPAIGNACCOUNTPLACEMENTINCLUSIONLISTASSOCIATION = 'CampaignAccountPlacementInclusionListAssociation'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of DownloadEntity from a JSON string"""
        return cls(json.loads(json_str))





from bingads.v13.internal.reporting.row_report import _RowReport
from bingads.v13.internal.reporting.xml_report import _XmlReport
from .report_contract import InvalidReportFormatException

class ReportFileReader:
    def __init__(self, file_path, format):
        if format is None:
            return
        self._file_path = file_path
        lower = format.lower()
        if lower == 'csv':
            self._report = _RowReport(file_path, format = 'Csv')
        elif lower == 'tsv':
            self._report = _RowReport(file_path, format = 'Tsv')
        elif lower == 'xml':
            self._report = _XmlReport(file_path)
        else:
            raise InvalidReportFormatException(format)

    def get_report(self):
        return self._report
    
    @property
    def file_path(self):
        """ The path of the bulk file to read.

        :rtype: str
        """

        return self._file_path
    
    def close(self):
        self._report.close()
        self._report = None


# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ProductPartitionPerformanceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    PRODUCTGROUP = 'ProductGroup'

    ADGROUPCRITERIONID = 'AdGroupCriterionId'

    PARTITIONTYPE = 'PartitionType'

    ADID = 'AdId'

    CURRENTMAXCPC = 'CurrentMaxCpc'

    CURRENCYCODE = 'CurrencyCode'

    DELIVEREDMATCHTYPE = 'DeliveredMatchType'

    BIDMATCHTYPE = 'BidMatchType'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    DEVICETYPE = 'DeviceType'

    LANGUAGE = 'Language'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ACCOUNTSTATUS = 'AccountStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    DESTINATIONURL = 'DestinationUrl'

    NETWORK = 'Network'

    TOPVSOTHER = 'TopVsOther'

    ASSISTS = 'Assists'

    REVENUE = 'Revenue'

    COSTPERASSIST = 'CostPerAssist'

    REVENUEPERCONVERSION = 'RevenuePerConversion'

    REVENUEPERASSIST = 'RevenuePerAssist'

    OFFERLANGUAGE = 'OfferLanguage'

    COUNTRYOFSALE = 'CountryOfSale'

    ADSTATUS = 'AdStatus'

    TRACKINGTEMPLATE = 'TrackingTemplate'

    CUSTOMPARAMETERS = 'CustomParameters'

    IMPRESSIONSHAREPERCENT = 'ImpressionSharePercent'

    IMPRESSIONLOSTTOBUDGETPERCENT = 'ImpressionLostToBudgetPercent'

    IMPRESSIONLOSTTORANKPERCENT = 'ImpressionLostToRankPercent'

    BENCHMARKBID = 'BenchmarkBid'

    BENCHMARKCTR = 'BenchmarkCtr'

    ADDISTRIBUTION = 'AdDistribution'

    CLICKTYPEID = 'ClickTypeId'

    TOTALCLICKSONADELEMENTS = 'TotalClicksOnAdElements'

    CLICKTYPE = 'ClickType'

    RETURNONADSPEND = 'ReturnOnAdSpend'

    BIDSTRATEGYTYPE = 'BidStrategyType'

    LOCALSTORECODE = 'LocalStoreCode'

    ASSISTEDIMPRESSIONS = 'AssistedImpressions'

    ASSISTEDCLICKS = 'AssistedClicks'

    CLICKSHAREPERCENT = 'ClickSharePercent'

    ABSOLUTETOPIMPRESSIONSHAREPERCENT = 'AbsoluteTopImpressionSharePercent'

    ASSISTEDCONVERSIONS = 'AssistedConversions'

    ALLCONVERSIONS = 'AllConversions'

    ALLREVENUE = 'AllRevenue'

    ALLCONVERSIONRATE = 'AllConversionRate'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    ALLRETURNONADSPEND = 'AllReturnOnAdSpend'

    ALLREVENUEPERCONVERSION = 'AllRevenuePerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ASSISTEDCONVERSIONSQUALIFIED = 'AssistedConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

    CAMPAIGNTYPE = 'CampaignType'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProductPartitionPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





from bingads.v13.internal.bulk.bulk_object import _BulkObject
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *


class BidSuggestionData:
    """ The best position, main line, and first page bid suggestion data corresponding to one keyword.

    If the requested DataScope includes BidSuggestionsData, the download will include bulk records corresponding to the
    properties of this class.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    """

    def __init__(self):
        self._best_position = None
        self._main_line = None
        self._first_page = None

    @property
    def best_position(self):
        """ Represents a best position bid suggestion that is derived from :class:`_BulkObject`.

        It can only be read from a bulk file by the :class:`.BulkFileReader` when reading the
        corresponding :class:`.BulkKeyword`. An instance of this class can represent
        a single best position bid, and thus one record in the bulk file. Properties of this class and of classes that
        it is derived from, correspond to fields of the Keyword Best Position Bid record in a bulk file.

        For more information, see Keyword Best Position Bid at https://go.microsoft.com/fwlink/?linkid=846127.

        :return: The best position bid suggestion.
        :rtype: BulkKeywordBidSuggestion
        """

        return self._best_position

    @best_position.setter
    def best_position(self, best_position):
        self._best_position = best_position

    @property
    def main_line(self):
        """ Represents a main line bid suggestion that is derived from :class:`_BulkObject`.

        It can only be read from a bulk file by the :class:`BulkFileReader` when reading the corresponding
        :class:`BulkKeyword`. An instance of this class can represent a single main line bid, and thus one record in
        the bulk file. Properties of this class and of classes that it is derived from, correspond to fields of the
        Keyword Main Line Bid record in a bulk file.

        For more information, see Keyword Main Line Bid at https://go.microsoft.com/fwlink/?linkid=846127.

        :return: The main line bid suggestion.
        :rtype: BulkKeywordBidSuggestion
        """

        return self._main_line

    @main_line.setter
    def main_line(self, main_line):
        self._main_line = main_line

    @property
    def first_page(self):
        """ Represents a first page bid suggestion that is derived from :class:`_BulkObject`. can only be read from a bulk
        file by the :class:`BulkFileReader` when reading the corresponding :class:`BulkKeyword`. An instance of this
        class can represent a single first page bid, and thus one record in the bulk file. Properties of this class and
        of classes that it is derived from, correspond to fields of the Keyword First Page Bid record in a bulk file.

        For more information, see Keyword First Page Bid at https://go.microsoft.com/fwlink/?linkid=846127.

        :return: The first page bid suggestion.
        :rtype: BulkKeywordBidSuggestion
        """

        return self._first_page

    @first_page.setter
    def first_page(self, first_page):
        self._first_page = first_page


class BulkKeywordBidSuggestion(_BulkObject):
    """ Represents a best position bid suggestion.

    It can only be read from a bulk file by the :class:`.BulkFileReader` when reading the corresponding :class:`.BulkKeyword`.
    An instance of this class can represent a single keyword bid position, and thus one record in the bulk file.
    """

    def __init__(self):
        self._keyword_text = None
        self._bid = None
        self._performance_data = None

    @property
    def keyword_text(self):
        """ The keyword corresponding to the suggested bid.

        Corresponds to the 'Keyword' field in the bulk file.
        :rtype: str
        """

        return self._keyword_text

    @property
    def bid(self):
        """ The suggested bid for the keyword.

        :rtype: float
        """

        return self._bid

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Keyword,
            field_to_csv=lambda c: bulk_str(c.keyword_text),
            csv_to_field=lambda c, v: setattr(c, '_keyword_text', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Bid,
            field_to_csv=lambda c: bulk_str(c.bid),
            csv_to_field=lambda c, v: setattr(c, '_bid', float(v) if v else None)
        ),
    ]

    def read_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkKeywordBidSuggestion._MAPPINGS)

    def write_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkKeywordBidSuggestion._MAPPINGS)

    @staticmethod
    def write_if_not_null(keyword_bid_suggestion, row_writer):
        if keyword_bid_suggestion is not None:
            row_writer.write_object_row(keyword_bid_suggestion)


class BulkKeywordBestPositionBid(BulkKeywordBidSuggestion):
    pass


class BulkKeywordFirstPageBid(BulkKeywordBidSuggestion):
    pass


class BulkKeywordMainLineBid(BulkKeywordBidSuggestion):
    pass



__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .bulk_conversion_goal import *
from .bulk_app_install_goal import *
from .bulk_event_goal import *
from .bulk_duration_goal import *
from .bulk_in_store_transaction_goal import *
from .bulk_in_store_visit_goal import *
from .bulk_offline_conversion_goal import *
from .bulk_pages_viewed_per_visit_goal import *
from .bulk_product_goal import *
from .bulk_url_goal import *



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ConversionGoalCategory(str, Enum):
    UNKNOWN = 'Unknown'

    NONE = 'None'

    PURCHASE = 'Purchase'

    ADDTOCART = 'AddToCart'

    BEGINCHECKOUT = 'BeginCheckout'

    SUBSCRIBE = 'Subscribe'

    SUBMITLEADFORM = 'SubmitLeadForm'

    BOOKAPPOINTMENT = 'BookAppointment'

    SIGNUP = 'Signup'

    REQUESTQUOTE = 'RequestQuote'

    GETDIRECTIONS = 'GetDirections'

    OUTBOUNDCLICK = 'OutboundClick'

    CONTACT = 'Contact'

    PAGEVIEW = 'PageView'

    DOWNLOAD = 'Download'

    OTHER = 'Other'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ConversionGoalCategory from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.billing.insertion_order_pending_changes import InsertionOrderPendingChanges
from openapi_client.models.billing.insertion_order_status import InsertionOrderStatus
from typing_extensions import Self

class InsertionOrder(BaseModel):
    """
    InsertionOrder
    """ # noqa: E501
    account_id: Optional[StrictStr] = Field(default=None, alias="AccountId")
    booking_country_code: Optional[StrictStr] = Field(default=None, alias="BookingCountryCode")
    comment: Optional[StrictStr] = Field(default=None, alias="Comment")
    end_date: Optional[datetime] = Field(default=None, alias="EndDate")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    last_modified_by_user_id: Optional[StrictStr] = Field(default=None, alias="LastModifiedByUserId")
    last_modified_time: Optional[datetime] = Field(default=None, alias="LastModifiedTime")
    notification_threshold: Optional[StrictFloat] = Field(default=None, alias="NotificationThreshold")
    reference_id: Optional[StrictStr] = Field(default=None, alias="ReferenceId")
    spend_cap_amount: Optional[StrictFloat] = Field(default=None, alias="SpendCapAmount")
    start_date: Optional[datetime] = Field(default=None, alias="StartDate")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    status: Optional[InsertionOrderStatus] = Field(default=None, alias="Status")
    purchase_order: Optional[StrictStr] = Field(default=None, alias="PurchaseOrder")
    pending_changes: Optional[InsertionOrderPendingChanges] = Field(default=None, alias="PendingChanges")
    account_number: Optional[StrictStr] = Field(default=None, alias="AccountNumber")
    budget_remaining: Optional[StrictFloat] = Field(default=None, alias="BudgetRemaining")
    budget_spent: Optional[StrictFloat] = Field(default=None, alias="BudgetSpent")
    budget_remaining_percent: Optional[StrictFloat] = Field(default=None, alias="BudgetRemainingPercent")
    budget_spent_percent: Optional[StrictFloat] = Field(default=None, alias="BudgetSpentPercent")
    series_name: Optional[StrictStr] = Field(default=None, alias="SeriesName")
    is_in_series: Optional[StrictBool] = Field(default=None, alias="IsInSeries")
    series_frequency_type: Optional[StrictStr] = Field(default=None, alias="SeriesFrequencyType")
    is_unlimited: Optional[StrictBool] = Field(default=None, alias="IsUnlimited")
    is_endless: Optional[StrictBool] = Field(default=None, alias="IsEndless")
    __properties: ClassVar[List[str]] = ["AccountId", "BookingCountryCode", "Comment", "EndDate", "Id", "LastModifiedByUserId", "LastModifiedTime", "NotificationThreshold", "ReferenceId", "SpendCapAmount", "StartDate", "Name", "Status", "PurchaseOrder", "PendingChanges", "AccountNumber", "BudgetRemaining", "BudgetSpent", "BudgetRemainingPercent", "BudgetSpentPercent", "SeriesName", "IsInSeries", "SeriesFrequencyType", "IsUnlimited", "IsEndless"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of pending_changes
        if self.pending_changes:
            _dict['PendingChanges'] = self.pending_changes.to_dict()
        # set to None if account_id (nullable) is None
        # and model_fields_set contains the field
        if self.account_id is None and "account_id" in self.model_fields_set:
            _dict['AccountId'] = None

        # set to None if booking_country_code (nullable) is None
        # and model_fields_set contains the field
        if self.booking_country_code is None and "booking_country_code" in self.model_fields_set:
            _dict['BookingCountryCode'] = None

        # set to None if comment (nullable) is None
        # and model_fields_set contains the field
        if self.comment is None and "comment" in self.model_fields_set:
            _dict['Comment'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if last_modified_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_by_user_id is None and "last_modified_by_user_id" in self.model_fields_set:
            _dict['LastModifiedByUserId'] = None

        # set to None if last_modified_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_time is None and "last_modified_time" in self.model_fields_set:
            _dict['LastModifiedTime'] = None

        # set to None if notification_threshold (nullable) is None
        # and model_fields_set contains the field
        if self.notification_threshold is None and "notification_threshold" in self.model_fields_set:
            _dict['NotificationThreshold'] = None

        # set to None if reference_id (nullable) is None
        # and model_fields_set contains the field
        if self.reference_id is None and "reference_id" in self.model_fields_set:
            _dict['ReferenceId'] = None

        # set to None if spend_cap_amount (nullable) is None
        # and model_fields_set contains the field
        if self.spend_cap_amount is None and "spend_cap_amount" in self.model_fields_set:
            _dict['SpendCapAmount'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if purchase_order (nullable) is None
        # and model_fields_set contains the field
        if self.purchase_order is None and "purchase_order" in self.model_fields_set:
            _dict['PurchaseOrder'] = None

        # set to None if pending_changes (nullable) is None
        # and model_fields_set contains the field
        if self.pending_changes is None and "pending_changes" in self.model_fields_set:
            _dict['PendingChanges'] = None

        # set to None if account_number (nullable) is None
        # and model_fields_set contains the field
        if self.account_number is None and "account_number" in self.model_fields_set:
            _dict['AccountNumber'] = None

        # set to None if budget_remaining (nullable) is None
        # and model_fields_set contains the field
        if self.budget_remaining is None and "budget_remaining" in self.model_fields_set:
            _dict['BudgetRemaining'] = None

        # set to None if budget_spent (nullable) is None
        # and model_fields_set contains the field
        if self.budget_spent is None and "budget_spent" in self.model_fields_set:
            _dict['BudgetSpent'] = None

        # set to None if budget_remaining_percent (nullable) is None
        # and model_fields_set contains the field
        if self.budget_remaining_percent is None and "budget_remaining_percent" in self.model_fields_set:
            _dict['BudgetRemainingPercent'] = None

        # set to None if budget_spent_percent (nullable) is None
        # and model_fields_set contains the field
        if self.budget_spent_percent is None and "budget_spent_percent" in self.model_fields_set:
            _dict['BudgetSpentPercent'] = None

        # set to None if series_name (nullable) is None
        # and model_fields_set contains the field
        if self.series_name is None and "series_name" in self.model_fields_set:
            _dict['SeriesName'] = None

        # set to None if is_in_series (nullable) is None
        # and model_fields_set contains the field
        if self.is_in_series is None and "is_in_series" in self.model_fields_set:
            _dict['IsInSeries'] = None

        # set to None if series_frequency_type (nullable) is None
        # and model_fields_set contains the field
        if self.series_frequency_type is None and "series_frequency_type" in self.model_fields_set:
            _dict['SeriesFrequencyType'] = None

        # set to None if is_unlimited (nullable) is None
        # and model_fields_set contains the field
        if self.is_unlimited is None and "is_unlimited" in self.model_fields_set:
            _dict['IsUnlimited'] = None

        # set to None if is_endless (nullable) is None
        # and model_fields_set contains the field
        if self.is_endless is None and "is_endless" in self.model_fields_set:
            _dict['IsEndless'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InsertionOrder from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "AccountId": obj.get("AccountId") if obj.get("AccountId") is not None else None,
                        "BookingCountryCode": obj.get("BookingCountryCode") if obj.get("BookingCountryCode") is not None else None,
                        "Comment": obj.get("Comment") if obj.get("Comment") is not None else None,
                        "EndDate": obj.get("EndDate") if obj.get("EndDate") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "LastModifiedByUserId": obj.get("LastModifiedByUserId") if obj.get("LastModifiedByUserId") is not None else None,
                        "LastModifiedTime": obj.get("LastModifiedTime") if obj.get("LastModifiedTime") is not None else None,
                        "NotificationThreshold": obj.get("NotificationThreshold") if obj.get("NotificationThreshold") is not None else None,
                        "ReferenceId": obj.get("ReferenceId") if obj.get("ReferenceId") is not None else None,
                        "SpendCapAmount": obj.get("SpendCapAmount") if obj.get("SpendCapAmount") is not None else None,
                        "StartDate": obj.get("StartDate") if obj.get("StartDate") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "PurchaseOrder": obj.get("PurchaseOrder") if obj.get("PurchaseOrder") is not None else None,
                        "PendingChanges": InsertionOrderPendingChanges.from_dict(obj["PendingChanges"]) if obj.get("PendingChanges") is not None else None,
                        "AccountNumber": obj.get("AccountNumber") if obj.get("AccountNumber") is not None else None,
                        "BudgetRemaining": obj.get("BudgetRemaining") if obj.get("BudgetRemaining") is not None else None,
                        "BudgetSpent": obj.get("BudgetSpent") if obj.get("BudgetSpent") is not None else None,
                        "BudgetRemainingPercent": obj.get("BudgetRemainingPercent") if obj.get("BudgetRemainingPercent") is not None else None,
                        "BudgetSpentPercent": obj.get("BudgetSpentPercent") if obj.get("BudgetSpentPercent") is not None else None,
                        "SeriesName": obj.get("SeriesName") if obj.get("SeriesName") is not None else None,
                        "IsInSeries": obj.get("IsInSeries") if obj.get("IsInSeries") is not None else None,
                        "SeriesFrequencyType": obj.get("SeriesFrequencyType") if obj.get("SeriesFrequencyType") is not None else None,
                        "IsUnlimited": obj.get("IsUnlimited") if obj.get("IsUnlimited") is not None else None,
                        "IsEndless": obj.get("IsEndless") if obj.get("IsEndless") is not None else None
        })
        return _obj
